@@ -28,8 +28,6 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <libhal.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <eel/eel-features.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
@@ -316,10 +314,10 @@ get_hal_context (void)
 }
 
 static gboolean
-_is_blank (GFileInfo *info)
+volume_is_blank (GVolume *volume)
 {
         LibHalContext *ctx;
-        const char    *udi;
+        char          *udi;
         gboolean       is_blank;
 
         ctx = get_hal_context ();
@@ -329,13 +327,14 @@ _is_blank (GFileInfo *info)
 
         is_blank = FALSE;
 
-        udi = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_MOUNTABLE_HAL_UDI);
+        udi = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
         if (udi != NULL) {
                 is_blank = libhal_device_get_property_bool (ctx,
                                                             udi,
                                                             "volume.disc.is_blank",
                                                             NULL);
         }
+        g_free (udi);
 
         return is_blank;
 }
@@ -361,10 +360,34 @@ drive_get_first_volume (GDrive *drive)
 }
 
 static gboolean
-_is_cd_device (GFileInfo *info)
+drive_is_cd_device (GDrive *drive)
 {
-        /* FIXME: */
-        return TRUE;
+        LibHalContext *ctx;
+        char          *udi;
+        gboolean       is_cd;
+
+        ctx = get_hal_context ();
+        if (ctx == NULL) {
+                return FALSE;
+        }
+
+        is_cd = FALSE;
+
+        udi = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+        if (udi != NULL) {
+                char *drive_type;
+                drive_type = libhal_device_get_property_string (ctx,
+                                                                udi,
+                                                                "storage.drive_type",
+                                                                NULL);
+                if (drive_type != NULL && strcmp (drive_type, "cdrom")) {
+                        is_cd = TRUE;
+                }
+                g_free (drive_type);
+        }
+        g_free (udi);
+
+        return is_cd;
 }
 
 static GList *
@@ -477,11 +500,11 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 
         if (drive != NULL
             && volume != NULL
-            && _is_cd_device (info)
-            && !_is_blank (info)) {
-                const char *device_path;
+            && drive_is_cd_device (drive)
+            && ! volume_is_blank (volume)) {
+                char *device_path;
 
-                device_path = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE);
+                device_path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 
                 item = nautilus_menu_item_new ("NautilusBurn::copy_disc",
                                                _("_Copy Disc..."),
@@ -493,6 +516,8 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
                 g_signal_connect (item, "activate",
                                   G_CALLBACK (copy_disc_activate_cb), NULL);
                 items = g_list_append (items, item);
+
+                g_free (device_path);
         }
 
         if (drive != NULL) {
